#!/usr/bin/env python3

import argparse
import json
import os
import shlex
import subprocess
import sys

from .conda import get_conda_variables
from .cuda import get_cuda_variables
from .macos import get_macos_variables
from .version import get_version_variables
from .wheel import get_wheel_variables


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser(description="Determine pytorch dependencies")
    parser.add_argument(
        "--package-type",
        help="Package type to lookup for",
        type=str,
        choices=["wheel", "conda"],
        # BUILD_TYPE for legacy scripts
        default=os.getenv("PACKAGE_TYPE", os.getenv("BUILD_TYPE", "wheel")),
    )
    parser.add_argument(
        "--channel",
        help="Channel to look in",
        choices=["nightly", "test"],
        type=str,
        default=os.getenv("CHANNEL", "nightly"),
    )
    parser.add_argument(
        "--platform",
        help="Platform to generate for",
        type=str,
        default=sys.platform,
    )
    parser.add_argument(
        "--gpu-arch-version",
        type=str,
        help="GPU arch version to look for",
        # CU_VERSION for legacy scripts
        default=os.getenv("GPU_ARCH_VERSION", os.getenv("CU_VERSION", "cpu")),
    )
    parser.add_argument(
        "--python-version",
        type=str,
        help="Python version to look for",
        default=os.getenv(
            "PYTHON_VERSION", ".".join([str(num) for num in sys.version_info[0:2]])
        ),
    )
    parser.add_argument(
        "--pytorch-version",
        type=str,
        help="PyTorch version to use",
        default=os.getenv("PYTORCH_VERSION", ""),
    )
    parser.add_argument(
        "--base-build-version",
        type=str,
        help="Base build version to use",
        # BUILD_VERSION for legacy scripts
        default=os.getenv("BUILD_VERSION", os.getenv("BASE_BUILD_VERSION", "")),
    )
    options = parser.parse_args()
    return options


def main():
    options = parse_args()
    variables = []
    if options.package_type == "conda":
        # TODO: Eventually it'd be nice to not have to rely on conda being installed
        output = subprocess.check_output(
            shlex.split(
                f"conda search --json 'pytorch[channel=pytorch-{options.channel}]'"
            ),
            stderr=subprocess.STDOUT,
        )
        conda_search = json.loads(output)
        variables.extend(
            get_conda_variables(
                conda_search=conda_search,
                platform=options.platform,
                gpu_arch_version=options.gpu_arch_version,
                python_version=options.python_version,
                pytorch_version=options.pytorch_version,
            )
        )
    elif options.package_type == "wheel":
        variables.extend(
            get_wheel_variables(
                platform=options.platform,
                gpu_arch_version=options.gpu_arch_version,
                python_version=options.python_version,
                pytorch_version=options.pytorch_version,
                channel=options.channel,
            )
        )
    if options.platform == "darwin":
        variables.extend(get_macos_variables())
    variables.extend(
        get_cuda_variables(
            options.package_type, options.platform, options.gpu_arch_version
        )
    )
    variables.extend(
        get_version_variables(
            package_type=options.package_type,
            channel=options.channel,
            gpu_arch_version=options.gpu_arch_version,
            base_build_version=options.base_build_version,
            platform=options.platform,
        )
    )
    for variable in sorted(variables):
        print(variable)


if __name__ == "__main__":
    main()



import re
import subprocess

from datetime import datetime
from pathlib import Path
from typing import List

LEADING_V_PATTERN = re.compile("^v")
TRAILING_RC_PATTERN = re.compile("-rc[0-9]*$")
LEGACY_BASE_VERSION_SUFFIX_PATTERN = re.compile("a0$")


class NoGitTagException(Exception):
    pass


def get_root_dir() -> Path:
    return Path(
        subprocess.check_output(["git", "rev-parse", "--show-toplevel"])
        .decode("ascii")
        .strip()
    )


def get_tag() -> str:
    root = get_root_dir()
    # We're on a tag
    am_on_tag = (
        subprocess.run(
            ["git", "describe", "--tags", "--exact"],
            cwd=root,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        ).returncode
        == 0
    )
    tag = ""
    if am_on_tag:
        dirty_tag = (
            subprocess.check_output(["git", "describe"], cwd=root)
            .decode("ascii")
            .strip()
        )
        # Strip leading v that we typically do when we tag branches
        # ie: v1.7.1 -> 1.7.1
        tag = re.sub(LEADING_V_PATTERN, "", dirty_tag)
        # Strip trailing rc pattern
        # ie: 1.7.1-rc1 -> 1.7.1
        tag = re.sub(TRAILING_RC_PATTERN, "", tag)
    return tag


def get_base_version() -> str:
    root = get_root_dir()
    try:
        dirty_version = open(root / "version.txt", "r").read().strip()
    except FileNotFoundError:
        print("# WARNING: Base version not found defaulting BUILD_VERSION to 0.1.0")
        dirty_version = "0.1.0"
    # Strips trailing a0 from version.txt, not too sure why it's there in the
    # first place
    return re.sub(LEGACY_BASE_VERSION_SUFFIX_PATTERN, "", dirty_version)


class PytorchVersion:
    def __init__(
        self,
        gpu_arch_version: str,
        no_build_suffix: bool,
        base_build_version: str,
    ) -> None:
        self.gpu_arch_version = gpu_arch_version
        self.no_build_suffix = no_build_suffix
        if base_build_version == "":
            base_build_version = get_base_version()
        self.base_build_version = base_build_version

    def get_post_build_suffix(self) -> str:
        if self.no_build_suffix:
            return ""
        return f"+{self.gpu_arch_version}"

    def get_release_version(self) -> str:
        if self.base_build_version:
            return f"{self.base_build_version}{self.get_post_build_suffix()}"
        if not get_tag():
            raise NoGitTagException(
                "Not on a git tag, are you sure you want a release version?"
            )
        return f"{get_tag()}{self.get_post_build_suffix()}"

    def get_nightly_version(self) -> str:
        date_str = datetime.today().strftime("%Y%m%d")
        build_suffix = self.get_post_build_suffix()
        return f"{self.base_build_version}.dev{date_str}{build_suffix}"


def get_version_variables(
    package_type: str,
    channel: str,
    gpu_arch_version: str,
    base_build_version: str,
    platform: str,
) -> List[str]:
    version = PytorchVersion(
        gpu_arch_version=gpu_arch_version,
        no_build_suffix=(platform == "darwin" or package_type == "conda"),
        base_build_version=base_build_version,
    )
    output_version = version.get_nightly_version()
    if channel == "test":
        output_version = version.get_release_version()
    return [f"export BUILD_VERSION='{output_version}'"]



import sys

from typing import List

from .utils import transform_cuversion

WINDOWS_PATH_PREFIX = "C:\\Program Files\\NVIDIA GPU Computing Toolkit\\CUDA\\v"


def get_cuda_arch_list(sanitized_version: str) -> str:
    if float(sanitized_version) > 11.3:
        return "3.5;5.0+PTX;6.0;7.0;7.5;8.0;8.6"
    else:  # mainly for cuda 10.2
        return "3.5;5.0+PTX;6.0;7.0;7.5"


def get_cuda_variables(
    package_type: str, platform: str, gpu_arch_version: str
) -> List[str]:
    version_suffix = ""
    pytorch_version_suffix = ""
    wheel_dir = ""
    if package_type == "wheel" and platform != "darwin":
        version_suffix = f"+{gpu_arch_version}"
        pytorch_version_suffix = f"+{gpu_arch_version}"
        wheel_dir = f"{gpu_arch_version}/"

    ret = [
        f"export VERSION_SUFFIX='{version_suffix}'",
        f"export PYTORCH_VERSION_SUFFIX='{pytorch_version_suffix}'",
        f"export WHEEL_DIR='{wheel_dir}'",
    ]

    cuda_home = ""
    sanitized_version = transform_cuversion(gpu_arch_version)
    # CUDA
    if gpu_arch_version.startswith("cu"):
        if platform.startswith("linux"):
            cuda_home = f"/usr/local/cuda-{sanitized_version}"
        elif platform in ("win32", "cygwin"):
            cuda_home = f"{WINDOWS_PATH_PREFIX}{sanitized_version}"
        else:
            raise NotImplementedError(
                f"Unrecognized platform ({sys.platform}) "
                f"for gpu_arch_version ({gpu_arch_version})"
            )
        cuda_arch_list = get_cuda_arch_list(sanitized_version)
        ret.extend(
            [
                f"export CUDA_HOME='{cuda_home}'",
                f"export TORCH_CUDA_ARCH_LIST='{cuda_arch_list}'",
                # Double quotes needed here to expand PATH var
                f'export PATH="{cuda_home}/bin:${{PATH}}"',
                "export FORCE_CUDA=1",
            ]
        )
    # ROCM
    elif gpu_arch_version.startswith("rocm"):
        ret.extend(
            [
                "export FORCE_CUDA=1",
            ]
        )
    return ret



import re

from typing import List

from .utils import transform_cuversion


def normalize_gpu_arch_version(gpu_arch_version: str):
    ver_one = gpu_arch_version
    ver_two = gpu_arch_version
    if gpu_arch_version != "cpu":
        ver_one = gpu_arch_version.replace("cu", "cuda")
        ver_two = (gpu_arch_version[:-1] + "." + gpu_arch_version[-1]).replace(
            "cu", "cuda"
        )
    return ver_one, ver_two


def get_conda_version_variables(
    conda_search: str, gpu_arch_version: str, python_version: str, pytorch_version: str
) -> List[str]:
    ver_one, ver_two = normalize_gpu_arch_version(gpu_arch_version)
    if pytorch_version == "":
        for pkg in reversed(conda_search["pytorch"]):
            if (
                any(
                    [
                        pkg["platform"] == "darwin",
                        ver_one in pkg["fn"],
                        ver_two in pkg["fn"],
                    ]
                )
                # matches the python version we're looking for
                and "py" + python_version in pkg["fn"]
            ):
                pytorch_version = re.sub(r"\\+.*$", "", pkg["version"])
                break
    return [
        f"export PYTORCH_VERSION='{pytorch_version}'",
        f"export CONDA_PYTORCH_BUILD_CONSTRAINT='- pytorch=={pytorch_version}'",
        f"export CONDA_PYTORCH_CONSTRAINT='- pytorch=={pytorch_version}'",
    ]


def get_conda_cuda_variables(platform: str, gpu_arch_version: str) -> List[str]:
    conda_build_variant = "cpu"
    cmake_use_cuda = "0"
    conda_cuda_toolkit_constraint = ""
    sanitized_version = transform_cuversion(gpu_arch_version)
    if sanitized_version != "cpu":
        conda_build_variant = "cuda"
        cmake_use_cuda = "1"
        if float(sanitized_version) >= 11.6:
            conda_cuda_toolkit_constraint = f"- pytorch-cuda={sanitized_version} # [not osx]"
        elif float(sanitized_version) == 11.3:
            conda_cuda_toolkit_constraint = "- cudatoolkit >=11.3,<11.4 # [not osx]"
        elif float(sanitized_version) == 10.2:
            conda_cuda_toolkit_constraint = "- cudatoolkit >=10.2,<10.3 # [not osx]"
        else:
            conda_cuda_toolkit_constraint = f"cudatoolkit={sanitized_version}"
    return [
        f"export CONDA_BUILD_VARIANT='{conda_build_variant}'",
        f"export CMAKE_USE_CUDA='{cmake_use_cuda}'",
        f"export CONDA_CUDATOOLKIT_CONSTRAINT='{conda_cuda_toolkit_constraint}'",
        f"export CUDATOOLKIT_CHANNEL=nvidia",
    ]


def get_conda_variables(
    conda_search: str,
    platform: str,
    gpu_arch_version: str,
    python_version: str,
    pytorch_version: str,
) -> List[str]:
    return get_conda_version_variables(
        conda_search, gpu_arch_version, python_version, pytorch_version
    ) + get_conda_cuda_variables(platform, gpu_arch_version)



from typing import List


def get_python_path_variables(python_version: str) -> List[str]:
    m = ""
    # For some reason python versions <= 3.7 require an m
    # probably better not to ask why
    if float(python_version) <= 3.7:
        m = "m"
    python_nodot = python_version.replace(".", "")
    python_abi = f"cp{python_nodot}-cp{python_nodot}{m}"
    return [f'export PATH="/opt/python/{python_abi}/bin:${{PATH}}"']


def get_pytorch_pip_install_command(
    gpu_arch_version: str,
    pytorch_version: str,
    channel: str,
) -> List[str]:
    torch_pkg = "torch"
    if pytorch_version != "":
        torch_pkg += f"=={pytorch_version}"
    pip_install = f"pip install {torch_pkg}"
    if channel == "nightly":
        pip_install += " --pre"
    extra_index = f"https://download.pytorch.org/whl/{channel}/{gpu_arch_version}"
    return [f"export PIP_INSTALL_TORCH='{pip_install} --extra-index-url {extra_index}'"]


def get_wheel_variables(
    platform: str,
    gpu_arch_version: str,
    python_version: str,
    pytorch_version: str,
    channel: str,
) -> List[str]:
    ret = []
    if platform.startswith("linux"):
        ret.extend(get_python_path_variables(python_version=python_version))
    ret.extend(
        get_pytorch_pip_install_command(
            gpu_arch_version=gpu_arch_version,
            pytorch_version=pytorch_version,
            channel=channel,
        )
    )
    return ret



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytorch_pkg_helpers']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['pytorch_pkg_helper = pytorch_pkg_helpers.__main__:main']}

setup_kwargs = {
    'name': 'pytorch-pkg-helpers',
    'version': '0.1.3',
    'description': 'Tool to determine pytorch dependencies for common domain library use cases',
    'long_description': 'None',
    'author': 'Eli Uriegas',
    'author_email': 'eliuriegas@fb.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



def transform_cuversion(gpu_arch_version: str) -> str:
    if gpu_arch_version.startswith("cu"):
        sanitized_version = gpu_arch_version.replace("cu", "")
        minor = sanitized_version[-1]
        major = sanitized_version[:-1]
        return f"{major}.{minor}"
    else:
        return gpu_arch_version
